/* not needed */
DROP TABLE WSI_STATISTICS4;
DROP TABLE WSI_STATISTICS6;

/* we want one protocol-independent key-values table */
CREATE TABLE WSI_KEYVALUES (
    PK INTEGER NOT NULL,
    KCLASS GENERAL_NAME COLLATE ISO8859_1,
    SUBCLASS GENERAL_NAME COLLATE ISO8859_1,
    KKEY GENERAL_NAME COLLATE ISO8859_1,
    KVALUE GENERAL_NAME COLLATE ISO8859_1,
    ENABLED BOOL,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

ALTER TABLE WSI_KEYVALUES ADD CONSTRAINT PK_KEYVALUES PRIMARY KEY(PK);
CREATE UNIQUE DESCENDING INDEX IDX_KV_A ON WSI_KEYVALUES(PK);
ALTER TABLE WSI_KEYVALUES ADD CONSTRAINT FK_KEYVALUES_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_WSI_KEYVALUES ON WSI_KEYVALUES (KKEY,KCLASS);

DROP TABLE WSI_KEYVALUES4;
DROP TABLE WSI_KEYVALUES6;

/* tftp policies */
CREATE TABLE TFTP_POLICY (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    PROTO GENERAL_NAME COLLATE ISO8859_1,
    ENABLED BOOL,
    SEQNO INTEGER,
    VERSION GENERAL_NAME COLLATE ISO8859_1,
    DATA LONG_TEXT,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

ALTER TABLE TFTP_POLICY ADD CONSTRAINT PK_TFTP_POL PRIMARY KEY (PK);
CREATE UNIQUE DESCENDING INDEX IDX_PKG_AA ON TFTP_POLICY(PK);
ALTER TABLE TFTP_POLICY ADD CONSTRAINT FK_PKGS_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE TFTP_POLICY ADD CONSTRAINT PK_PKGS_NAME UNIQUE (NAME);
CREATE INDEX IDX_TFTP_POL_SEQNO ON TFTP_POLICY (SEQNO);
CREATE INDEX IDX_TFTP_POL_ENABLED ON TFTP_POLICY (ENABLED);
CREATE INDEX IDX_TFTP_POL_VERSION ON TFTP_POLICY (VERSION);

/* remove the old -1 vendor (device type) id, reserve that for new type id 'Unknown Type' */
DELETE FROM DHCP_VENDOR_ATTR4 WHERE (VID = '-1/1');

/* delete the old -1/n vendor-specific options - the vcinc4 file will insert the new ones */
DELETE FROM DHCP_OPTION_ATTR4 WHERE (VENDOR_ID = '-1/1');

/* New column to dictate the time at which the device configuration should be invalidated. 0 = never */
ALTER TABLE WSI_ACCOUNTS ADD RECONFIG_TIME TIME_T;
/* New column to dictate the time at which device ACLs should be invalidated. 0 = never */
ALTER TABLE WSI_ACCOUNTS ADD REPROV_TIME TIME_T;

/* add metadata columns to accounts and domains */
ALTER TABLE WSI_ACCOUNTS ADD MDATA BLOB sub_type 1 segment size 1;
ALTER TABLE WSI_DOMAINS ADD MDATA BLOB sub_type 1 segment size 1;

ALTER TABLE DHCP_ABINDINGS4 ADD LEASE_EXPIRE TIME_T;
ALTER TABLE DHCP_ABINDINGS6 ADD LEASE_EXPIRE TIME_T;

ALTER TABLE DHCP_NBINDINGS4 ADD LEASE_EXPIRE TIME_T;
ALTER TABLE DHCP_NBINDINGS6 ADD LEASE_EXPIRE TIME_T;

ALTER TABLE DHCP_NPOOLS4 ADD ADDR_RANGE FOREIGN_KEY_ID;
ALTER TABLE DHCP_NPOOLS4 ADD ADDR_RANGE_CL CLASS_NAME COLLATE ISO8859_1;
ALTER TABLE DHCP_NPOOLS4 ADD CONSTRAINT FK_NPOOLS4_RANGE FOREIGN KEY (ADDR_RANGE) REFERENCES DHCP_ADDRESS_RANGE (PK);

ALTER TABLE DHCP_NPOOLS6 ADD ADDR_RANGE FOREIGN_KEY_ID;
ALTER TABLE DHCP_NPOOLS6 ADD ADDR_RANGE_CL CLASS_NAME COLLATE ISO8859_1;
ALTER TABLE DHCP_NPOOLS6 ADD CONSTRAINT FK_NPOOLS6_RANGE FOREIGN KEY (ADDR_RANGE) REFERENCES DHCP_ADDRESS_RANGE (PK);

ALTER TABLE DHCP_AFBINDINGS4 DROP LEASE_EXP_TIME;

CREATE INDEX IDX_ABINDING4_LE ON DHCP_ABINDINGS4 (LEASE_EXPIRE);
CREATE INDEX IDX_ABINDING6_LE ON DHCP_ABINDINGS6 (LEASE_EXPIRE);
CREATE INDEX IDX_NBINDING4_LE ON DHCP_NBINDINGS4 (LEASE_EXPIRE);
CREATE INDEX IDX_NBINDING6_LE ON DHCP_NBINDINGS6 (LEASE_EXPIRE);

/* remove all ACLs for accounts 1,2 and 3 since the all nodes accounts (all nodes, all v4 nodes, all v6 nodes) are no longer explicit */
DELETE FROM WSI_ACCESS_CONTROL WHERE (ACCESS_ID = 1);
DELETE FROM WSI_ACCESS_CONTROL WHERE (ACCESS_ID = 2);
DELETE FROM WSI_ACCESS_CONTROL WHERE (ACCESS_ID = 3);

/* now delete the explicit accounts for all nodes */
DELETE FROM WSI_ACCOUNTS WHERE (ID = 1);
DELETE FROM WSI_ACCOUNTS WHERE (ID = 2);
DELETE FROM WSI_ACCOUNTS WHERE (ID = 3);

/* Now change domain id 3 to be domain id 2 and make it protocol agnostic */
UPDATE WSI_DOMAINS SET NAME = 'All devices', DESCRIPTION = 'All devices on your network', ID = 2 WHERE (ID = 3);

/* Change 'Admin' to 'Administrators', 'All users' to 'All operators' */
UPDATE WSI_DOMAINS SET NAME = 'Administrators' WHERE (ID = 0);
UPDATE WSI_DOMAINS SET NAME = 'All operators' WHERE (ID = 1);

/* all secured objects referring to domain 3 or 4 should now refer to domain 2 */
UPDATE WSI_DOMAIN_ID SET DOMAIN_ID = 2 WHERE (DOMAIN_ID = 3);
UPDATE WSI_DOMAIN_ID SET DOMAIN_ID = 2 WHERE (DOMAIN_ID = 4);

/* update name to id mapping for the new all nodes domain */
UPDATE WSI_DOMAIN_NAME_ID SET ID = 2 WHERE (ID = 3);

/* in case there are still any access controls referring to domain id 3 or 4, just delete them since they're fully implicit */
DELETE FROM WSI_ACCESS_CONTROL WHERE (DOMAIN_ID = 3);
DELETE FROM WSI_ACCESS_CONTROL WHERE (DOMAIN_ID = 4);

/* now delete the 'all v6 nodes' domain altogether it should no longer be referenced */
DELETE FROM WSI_DOMAINS WHERE (ID = 4);
DELETE FROM WSI_DOMAIN_NAME_ID WHERE (ID = 4);

UPDATE WSI_DOMAIN_MSHIPS_LINKS SET DEST_SEQ_KEY = 2 WHERE (DEST_SEQ_KEY = 3);
UPDATE WSI_DOMAIN_MSHIPS_LINKS SET DEST_SEQ_KEY = 2 WHERE (DEST_SEQ_KEY = 4);

/* Create a domain group table. */
CREATE TABLE WSI_DOMAIN_GROUPS (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    ID DOM_ID_TYPE,
    DOM_MEMBERS FOREIGN_KEY_ID,
    DOM_MEMBERS_SC SELECT_CRITERIA COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE WSI_BI_DOMGRP_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    BI_SRC_REF FOREIGN_KEY_ID,
    BI_DEST_REF FOREIGN_KEY_ID,
    BI_SRC_REF_CL CLASS_NAME COLLATE ISO8859_1,
    BI_DEST_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE,
  DEST_EXT_SEQ_KEY INDEX_TYPE);

CREATE DOMAIN ULONG_64 AS NUMERIC(8,0)
DEFAULT 0;

CREATE DOMAIN VALID_GENERAL_NAME AS VARCHAR (80) CHARACTER SET ISO8859_1
CHECK (VALUE NOT IN (''));

ALTER TABLE WSI_DOMAIN_GROUPS ADD CONSTRAINT PK_WSI_DOMAIN_GROUPS PRIMARY KEY (PK);
ALTER TABLE WSI_DOMAIN_GROUPS ADD CONSTRAINT WSI_DOMAIN_GROUPS_NAME UNIQUE (NAME);
ALTER TABLE WSI_BI_DOMGRP_LINKS ADD CONSTRAINT PK_BI_DOMGRP_LINKS PRIMARY KEY (PK);
ALTER TABLE WSI_DOMAIN_GROUPS ADD CONSTRAINT FK_WSI_DOMAIN_GROUPS_SECURE FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE WSI_BI_DOMGRP_LINKS ADD CONSTRAINT FK_BI_DOMGRP_LINKS_DEST FOREIGN KEY (BI_DEST_REF) REFERENCES WSI_DOMAINS (PK);
ALTER TABLE WSI_BI_DOMGRP_LINKS ADD CONSTRAINT FK_BI_DOMGRP_LINKS_SRC FOREIGN KEY (BI_SRC_REF) REFERENCES WSI_DOMAIN_GROUPS(PK);
ALTER TABLE WSI_DOMAINS ADD GROUPDOM FOREIGN_KEY_ID;
ALTER TABLE WSI_DOMAINS ADD GROUPDOM_SC SELECT_CRITERIA COLLATE ISO8859_1;
ALTER TABLE WSI_ACCOUNTS ADD SUBSCRIBER_ID ULONG_64;
ALTER TABLE WSI_ACCOUNTS DROP CONSTRAINT PK_ACCOUNTS_COMPOUND_UNIQUE;
/* Domain classes are no longer used */
ALTER TABLE WSI_DOMAINS DROP CONSTRAINT PK_DOMAINS_COMPOUND_UNIQUE;
ALTER TABLE WSI_DOMAINS DROP DCLASS;
ALTER TABLE WSI_ACCOUNTS ALTER COLUMN NAME TYPE VALID_GENERAL_NAME;
ALTER TABLE WSI_DOMAINS ALTER COLUMN NAME TYPE VALID_GENERAL_NAME;
ALTER TABLE WSI_ACCOUNTS ADD CONSTRAINT PK_ACCOUNTS_COMPOUND_UNIQUE UNIQUE (NAME,ACLASS);
ALTER TABLE WSI_DOMAINS ADD CONSTRAINT PK_DOMAINS_NAME_UNIQUE UNIQUE (NAME);
